# (c) 2011-2012 Cloudera, Inc. All rights reserved.

import os
import shutil
try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor import MonitorDaemon, schema
from cmf.monitor import constants
from cmf.monitor.journalnode import JournalNodeMonitor
from cmf.monitor.journalnode.json_parser_jmx import SAMPLE
from cmf.monitor.tests import make_config_file

from nose.tools import assert_true, assert_equals, assert_false

class MockJournalNodeMonitor(JournalNodeMonitor):
  def __init__(self, daemon, registry, string):
    JournalNodeMonitor.__init__(self, daemon, registry)
    self.set_string(string)
    self.set_throw_error(None)

  def set_string(self, string):
    self._string = string

  def set_throw_error(self, error):
    self._error = error

  def urlopen(self, url):
    if self._error:
      raise self._error
    return StringIO(self._string)

# CM 4.1 config file
def _make_cm41_format_config_file(version, dfs_journalnode_http_port):
  properties = """[journalnode]
role_name = hdfs-JN
service_name = my_hdfs
service_version = %s
host = localhost
dfs_journalnode_http_port = %s
dfs_journalnode_edits_dir = /path/to/edits/dir
log_dir = /path/to/logs
collect_interval = 1.0
"""
  return  make_config_file(properties % (version, dfs_journalnode_http_port,))

def test_jn_lifecycle_cdh41():
  do_jn_lifecycle(4, _make_cm41_format_config_file, SAMPLE)

def do_jn_lifecycle(version, make_config_file_function, json_sample):
  daemon = MonitorDaemon("host", "host", agent=None)
  reg = daemon.get_registry_for_testing()
  monitor = MockJournalNodeMonitor(daemon, reg, json_sample)
  monitor.refresh_pid(os.getpid(), True)

  # Make two different configs
  conf_dir = make_config_file_function(version, 50075)
  conf_dir2 = make_config_file_function(version, 50076)

  try:
    assert_equals(0, len(reg.get_registry()))

    # Start it
    monitor.refresh(conf_dir)
    assert_equals(monitor, reg.get_registry().pop())
    monitor.collect()
    assert_true('50075' in monitor.metrics_url, "Config installed")

    # Refresh with a different config
    monitor.refresh(conf_dir2)
    assert_equals(monitor, reg.get_registry().pop())
    assert_equals(1, len(reg.get_registry()))

    monitor.collect()
    assert_true('50076' in monitor.metrics_url, "Using the new config")

    # Test metrics and role update
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('hdfs-JN', role_update['rolename'])
    assert_equals('my_hdfs', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_true(metric_dict[schema.TOTAL_CPU_SYSTEM] >= 0.0) 
    assert_equals(888.9375, metric_dict[schema.JVM_MAX_MEMORY_MB])
    assert_equals(1, metric_dict[schema.JVM_BLOCKED_THREADS])
    assert_equals(2, metric_dict[schema.JVM_NEW_THREADS])
    assert_equals(3, metric_dict[schema.JVM_RUNNABLE_THREADS])
    assert_equals(4, metric_dict[schema.JVM_TERMINATED_THREADS])
    assert_equals(5, metric_dict[schema.JVM_TIMED_WAITING_THREADS])
    assert_equals(6, metric_dict[schema.JVM_WAITING_THREADS])
    assert_equals(21, metric_dict[schema.JVM_TOTAL_THREADS])
    assert_equals(constants.WEB_METRICS_COLLECTION_GOOD,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)
    # metrics from detailed_rpc
    assert_true(schema.ACCEPT_RECOVERY_AVG_TIME in metric_dict)
    assert_true(schema.ACCEPT_RECOVERY_NUM_OPS in metric_dict)
    assert_true(schema.FINALIZE_LOG_SEGMENT_AVG_TIME in metric_dict)
    assert_true(schema.FINALIZE_LOG_SEGMENT_NUM_OPS in metric_dict)
    assert_true(schema.GET_EDIT_LOG_MANIFEST_AVG_TIME in metric_dict)
    assert_true(schema.GET_EDIT_LOG_MANIFEST_NUM_OPS in metric_dict)
    assert_true(schema.GET_JOURNAL_STATE_AVG_TIME in metric_dict)
    assert_true(schema.GET_JOURNAL_STATE_NUM_OPS in metric_dict)
    assert_true(schema.JOURNAL_AVG_TIME in metric_dict)
    assert_true(schema.JOURNAL_NUM_OPS in metric_dict)
    assert_true(schema.NEW_EPOCH_AVG_TIME in metric_dict)
    assert_true(schema.NEW_EPOCH_NUM_OPS in metric_dict)
    assert_true(schema.PREPARE_RECOVERY_AVG_TIME in metric_dict)
    assert_true(schema.PREPARE_RECOVERY_NUM_OPS in metric_dict)
    assert_true(schema.START_LOG_SEGMENT_AVG_TIME in metric_dict)
    assert_true(schema.START_LOG_SEGMENT_NUM_OPS in metric_dict)
    # metrics from ugi
    assert_true(schema.LOGIN_FAILURE_AVG_TIME in metric_dict)
    assert_true(schema.LOGIN_FAILURE_NUM_OPS in metric_dict)
    assert_true(schema.LOGIN_SUCCESS_AVG_TIME in metric_dict)
    assert_true(schema.LOGIN_SUCCESS_NUM_OPS in metric_dict)
    # metrics from journal
    assert_true(schema.BATCHES_WRITTEN in metric_dict)
    assert_true(schema.BATCHES_WRITTEN_WHILE_LAGGING in metric_dict)
    assert_true(schema.BYTES_WRITTEN in metric_dict)
    assert_true(schema.CURRENT_LAG_TXNS_JOURNALNODE in metric_dict)
    assert_true(schema.LAST_PROMISED_EPOCH in metric_dict)
    assert_true(schema.LAST_WRITER_EPOCH in metric_dict)
    assert_true(schema.LAST_WRITTEN_TX_ID in metric_dict)
    assert_true(schema.SYNCS300S50TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS300S75TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS300S90TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS300S95TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS300S99TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS300S_NUM_OPS in metric_dict)
    assert_true(schema.SYNCS3600S50TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS3600S75TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS3600S90TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS3600S95TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS3600S99TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS3600S_NUM_OPS in metric_dict)
    assert_true(schema.SYNCS60S50TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS60S75TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS60S90TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS60S95TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS60S99TH_PERCENTILE_LATENCY_MICROS in metric_dict)
    assert_true(schema.SYNCS60S_NUM_OPS in metric_dict)
    assert_true(schema.TXNS_WRITTEN in metric_dict)
    # metrics from rpc
    assert_true(schema.RPC_CALL_QUEUE_LENGTH in metric_dict)
    assert_true(schema.RPC_NUM_OPEN_CONNECTIONS in metric_dict)
    assert_true(schema.RPC_RECEIVED_BYTES in metric_dict)
    assert_true(schema.RPC_AUTHENTICATION_FAILURES in metric_dict)
    assert_true(schema.RPC_AUTHENTICATION_SUCCESSES in metric_dict)
    assert_true(schema.RPC_AUTHORIZATION_FAILURES in metric_dict)
    assert_true(schema.RPC_AUTHORIZATION_SUCCESSES in metric_dict)
    assert_true(schema.RPC_PROCESSING_TIME_AVG_TIME in metric_dict)
    assert_true(schema.RPC_PROCESSING_TIME_NUM_OPS in metric_dict)
    assert_true(schema.RPC_QUEUE_TIME_AVG_TIME in metric_dict)
    assert_true(schema.RPC_QUEUE_TIME_NUM_OPS in metric_dict)
    assert_true(schema.RPC_SENT_BYTES in metric_dict)
    # metrics from jvm
    assert_true(schema.JVM_GC_COUNT in metric_dict)
    assert_true(schema.JVM_GC_TIME_MS in metric_dict)
    assert_true(schema.LOG_ERROR in metric_dict)
    assert_true(schema.LOG_FATAL in metric_dict)
    assert_true(schema.LOG_INFO in metric_dict)
    assert_true(schema.LOG_WARN in metric_dict)
    assert_true(schema.JVM_HEAP_COMMITTED_MB in metric_dict)
    assert_true(schema.JVM_HEAP_USED_MB in metric_dict)
    assert_true(schema.JVM_BLOCKED_THREADS in metric_dict)
    assert_true(schema.JVM_NEW_THREADS in metric_dict)
    assert_true(schema.JVM_RUNNABLE_THREADS in metric_dict)
    assert_true(schema.JVM_TERMINATED_THREADS in metric_dict)
    assert_true(schema.JVM_TIMED_WAITING_THREADS in metric_dict)
    assert_true(schema.JVM_WAITING_THREADS in metric_dict)
    # metrics from metrics_stats
    assert_true(schema.METRICS_DROPPED_PUB_ALL in metric_dict)
    assert_true(schema.METRICS_NUM_ACTIVE_SINKS in metric_dict)
    assert_true(schema.METRICS_NUM_ACTIVE_SOURCES in metric_dict)
    assert_true(schema.METRICS_NUM_ALL_SINKS in metric_dict)
    assert_true(schema.METRICS_NUM_ALL_SOURCES in metric_dict)
    assert_true(schema.METRICS_PUBLISH_AVG_TIME in metric_dict)
    assert_true(schema.METRICS_PUBLISH_NUM_OPS in metric_dict)
    assert_true(schema.METRICS_SNAPSHOT_AVG_TIME in metric_dict)
    assert_true(schema.METRICS_SNAPSHOT_NUM_OPS in metric_dict)

    # Test fs update
    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == 2, "We configured 2 dirs: %s" % dir_metrics)
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_equals('/path/to/logs', dir_metrics[0]['path'])
    assert_equals('/path/to/edits/dir', dir_metrics[1]['path'])

    # Test when the process is not running
    monitor.refresh_pid(None)
    monitor.collect()

    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('hdfs-JN', role_update['rolename'])
    assert_equals('my_hdfs', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']

    dir_metrics = role_update['dirUpdates']
    assert_true(len(dir_metrics) == 2, "We configured 2 dirs: %s" % dir_metrics)
    assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
    assert_equals('/path/to/logs', dir_metrics[0]['path'])
    assert_equals('/path/to/edits/dir', dir_metrics[1]['path'])

    # Test with a collection error
    monitor.refresh_pid(os.getpid(), True)
    monitor.set_throw_error(IOError("BOOM!"))
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_COMMUNICATION_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Test with some invalid JSON
    monitor.set_throw_error(None)
    monitor.set_string("THIS IS BOGUS JSON!")
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_PARSE_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Stop it
    monitor.stop()
    assert_equals(0, len(reg.get_registry()))
  finally:
    shutil.rmtree(conf_dir)
    shutil.rmtree(conf_dir2)
