// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/cmf/page/ServiceChartSet",
	"cloudera/chart/spec/MockAnyChartStock"
], function(ServiceChartSet, MockAnyChartStock) {

describe("ServiceChartSetSpec Tests", function() {
	var chartSet;

	beforeEach(function() {
		var params = {
			'startDate': new Date(1312244796541),
			'endDate': new Date(1312246596541),
			'urlParams': {
				'key': 'foo+1'
			},
			'timeSeriesDescriptors': [ {
				"context" : {
					"id" : "myRoleName",
					"type" : "ROLE"
				},
				"metric" : {
					"id" : 10011,
					"context" : "ROLE",
					"name" : "dfs_capacity_used",
					"desc" : "Storage space used by HDFS",
					"title" : "HDFS size",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "10011",
					"title" : "HDFS size",
					"discrete" : false
				}
			}, {
				"context" : {
					"id" : "myRoleName2",
					"type" : "ROLE"
				},
				"metric" : {
					"id" : 10100,
					"context" : "ROLE",
					"name" : "nn_ops_read",
					"desc" : "Number of namespace read operations",
					"title" : "Namespace reads",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "10100",
					"title" : "Namespace reads",
					"discrete" : false
				}
			} ],
			'jsonURL': "/cmf/monitor/service_chart_json"
		};
		chartSet = new ServiceChartSet(params, "testId");
		// console.log(chartSet.getConfig());
	});

	afterEach(function() {
		chartSet.unsubscribe();
	});

	describe("when a ServiceChartSet is created", function() {
		var config;
		beforeEach(function() {
			jasmine.Ajax.useMock();
			config = chartSet.getConfig();
		});

		it("should see 10 generalDataProviders.", function() {
			// We expect the configuration to have the following data providers.
			expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
			expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
			expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");

			expect(config.data.dataProviders.generalDataProviders[1].dataSet).toEqual("dataSet_1");
			expect(config.data.dataProviders.generalDataProviders[1].id).toEqual("dp_1");
		});

		it("should see 11 dataSets.", function() {
			expect(config.data.dataSets.length).toEqual(11);
			expect(config.data.dataSets[0].id).toEqual("dataSet_0");
			expect(config.data.dataSets[1].id).toEqual("dataSet_1");
			expect(config.data.dataSets[10].id).toEqual("chartStretcher");
		});

		it("should see 2 charts.", function() {
			expect(config.settings.charts.length).toEqual(2);
		});

		it("should see a time series on chart 1.", function() {
			expect(config.settings.charts[0].id).toEqual("10011");
			expect(config.settings.charts[0].seriesList.length).toEqual(1);
			expect(config.settings.charts[0].seriesList[0].dataProvider).toEqual("dp_0");
			expect(config.settings.charts[0].seriesList[0].id).toEqual("series_ROLE_myRoleName_10011");
			expect(config.settings.charts[0].seriesList[0].name).toEqual("HDFS size");
			expect(config.settings.charts[0].seriesList[0].type).toEqual("Line");
		});

		it("should see a time series on chart 2.", function() {
			expect(config.settings.charts[1].id).toEqual("10100");
			expect(config.settings.charts[1].seriesList.length).toEqual(1);
			expect(config.settings.charts[1].seriesList[0].dataProvider).toEqual("dp_1");
			expect(config.settings.charts[1].seriesList[0].id).toEqual("series_ROLE_myRoleName2_10100");
			expect(config.settings.charts[1].seriesList[0].name).toEqual("Namespace reads");
			expect(config.settings.charts[1].seriesList[0].type).toEqual("Line");
		});

		it("should format the chart post data correctly.", function() {
			var actual = chartSet._generateChartsPostData(new Date(5), new Date(10));
			var expected = "key=foo%2B1&startTime=5&endTime=10&metrics[0].metricId=10011&metrics[0].derivativeType=undefined&metrics[0].derivativeScale=undefined&metrics[0].units=undefined&metrics[0].contextName=myRoleName&metrics[0].contextType=ROLE&metrics[1].metricId=10100&metrics[1].derivativeType=undefined&metrics[1].derivativeScale=undefined&metrics[1].units=undefined&metrics[1].contextName=myRoleName2&metrics[1].contextType=ROLE";
			expect(actual).toEqual(expected);
		});

		it("should format the event post data correctly.", function() {
			var actual = chartSet._generateEventsPostData(new Date(5), new Date(10));
			var expected = "key=foo%2B1&timeUpper=10&timeLower=5&role=myRoleName2&summarise=on&summariseSize=0.5&summariseGroupBy=CATEGORY&summariseBy=SEVERITY";
			expect(actual).toEqual(expected);
		});

	});
});
});

