// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util",
	"cloudera/cmon/page/ActivityChartSet",
	"cloudera/chart/spec/MockAnyChartStock"
], function(Util, ActivityChartSet, MockAnyChartStock) {

describe("ActivityChartSet Tests", function() {
	var chartSet;

	beforeEach(function() {
		var params = {
			'startDate': new Date(1311915938866),
			'endDate': new Date(1311917738866),
			'chartDescriptors': [ {
				"metric" : {
					"id" : 3,
					"context" : "CLUSTER",
					"name" : "maps_running",
					"desc" : "Running map tasks",
					"title" : "Running maps",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "CLUSTER_3",
					"title" : "Running maps",
					"discrete" : true
				}
			}, {
				"metric" : {
					"id" : 5,
					"context" : "CLUSTER",
					"name" : "reduces_running",
					"desc" : "Running reduce tasks",
					"title" : "Running reducers",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "CLUSTER_5",
					"title" : "Running reducers",
					"discrete" : true
				}
			}, {
				"metric" : {
					"id" : 3001,
					"context" : "ACTIVITY",
					"name" : "total_launched_maps",
					"desc" : "Total launched map attempts",
					"title" : "Launched maps",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "ACTIVITY_3001",
					"title" : "Launched maps",
					"discrete" : true
				}
			}, {
				"metric" : {
					"id" : 3004,
					"context" : "ACTIVITY",
					"name" : "total_launched_reduces",
					"desc" : "Total launched reduce attempts",
					"title" : "Launched reducers",
					"selector" : "AVERAGE"
				},
				"chart" : {
					"id" : "ACTIVITY_3004",
					"title" : "Launched reducers",
					"discrete" : true
				}
			} ],
			'contexts': [ {
				"id" : "cluster",
				"type" : "CLUSTER"
			} ]
		};
		chartSet = new ActivityChartSet(params, "testId");
		// console.log(chartSet.getConfig());
	});

	describe("when an ActivityChartSet is created", function() {
		var config;
		beforeEach(function() {
			jasmine.Ajax.useMock();
			config = chartSet.getConfig();
		});

		it("should see 10 generalDataProviders.", function() {
			// We expect the configuration to have the following data providers.
			expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
			expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
			expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");

			expect(config.data.dataProviders.generalDataProviders[1].dataSet).toEqual("dataSet_1");
			expect(config.data.dataProviders.generalDataProviders[1].id).toEqual("dp_1");
		});

		it("should see 11 dataSets.", function() {
			expect(config.data.dataSets.length).toEqual(11);
			expect(config.data.dataSets[0].id).toEqual("dataSet_0");
			expect(config.data.dataSets[1].id).toEqual("dataSet_1");
			expect(config.data.dataSets[10].id).toEqual("chartStretcher");
		});

		it("should see 4 charts.", function() {
			expect(config.settings.charts.length).toEqual(4);
		});

		it("should see a time series on chart 1.", function() {
			expect(config.settings.charts[0].id).toEqual("CLUSTER_3");
			expect(config.settings.charts[0].seriesList.length).toEqual(1);
			expect(config.settings.charts[0].seriesList[0].dataProvider).toEqual("dp_0");
			expect(config.settings.charts[0].seriesList[0].id).toEqual("series_CLUSTER_cluster_3");
			expect(config.settings.charts[0].seriesList[0].name).toEqual("cluster Running maps");
			expect(config.settings.charts[0].seriesList[0].type).toEqual("StepLine");
		});

		it("should see a time series on chart 2.", function() {
			expect(config.settings.charts[1].id).toEqual("CLUSTER_5");
			expect(config.settings.charts[1].seriesList.length).toEqual(1);
			expect(config.settings.charts[1].seriesList[0].dataProvider).toEqual("dp_1");
			expect(config.settings.charts[1].seriesList[0].id).toEqual("series_CLUSTER_cluster_5");
			expect(config.settings.charts[1].seriesList[0].name).toEqual("cluster Running reducers");
			expect(config.settings.charts[1].seriesList[0].type).toEqual("StepLine");
		});

		it("should see any empty chart 3.", function() {
			expect(config.settings.charts[2].id).toEqual("ACTIVITY_3001");
			expect(config.settings.charts[2].seriesList.length).toEqual(0);
		});

		it("should see any empty chart 4.", function() {
			expect(config.settings.charts[3].id).toEqual("ACTIVITY_3004");
			expect(config.settings.charts[3].seriesList.length).toEqual(0);
		});

		describe("ActivityChartSet.setContext Tests - Addition", function() {
			beforeEach(function() {
				jasmine.Ajax.useMock();
				chartSet.setContexts([{
					"id": "cluster",
					"type": "CLUSTER"
				}, {
					"id": "job1",
					"type": "ACTIVITY"
				}]);
			});

			describe("when a new job is added", function() {
				it("should see 10 generalDataProviders.", function() {
					// We expect the configuration to have the following data providers.
					expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
					expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
					expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");

					expect(config.data.dataProviders.generalDataProviders[1].dataSet).toEqual("dataSet_1");
					expect(config.data.dataProviders.generalDataProviders[1].id).toEqual("dp_1");

					expect(config.data.dataProviders.generalDataProviders[2].dataSet).toEqual("dataSet_2");
					expect(config.data.dataProviders.generalDataProviders[2].id).toEqual("dp_2");

					expect(config.data.dataProviders.generalDataProviders[3].dataSet).toEqual("dataSet_3");
					expect(config.data.dataProviders.generalDataProviders[3].id).toEqual("dp_3");
				});

				it("should see 11 dataSets.", function() {
					expect(config.data.dataSets.length).toEqual(11);
					expect(config.data.dataSets[0].id).toEqual("dataSet_0");
					expect(config.data.dataSets[1].id).toEqual("dataSet_1");
					expect(config.data.dataSets[2].id).toEqual("dataSet_2");
					expect(config.data.dataSets[3].id).toEqual("dataSet_3");
					expect(config.data.dataSets[10].id).toEqual("chartStretcher");
				});

				it("should see 4 charts.", function() {
					expect(config.settings.charts.length).toEqual(4);
				});

				it("should still see a time series on chart 1.", function() {
					expect(config.settings.charts[0].id).toEqual("CLUSTER_3");
					expect(config.settings.charts[0].seriesList.length).toEqual(1);
				});

				it("should still see a time series on chart 2.", function() {
					expect(config.settings.charts[1].id).toEqual("CLUSTER_5");
					expect(config.settings.charts[1].seriesList.length).toEqual(1);
					expect(config.settings.charts[1].seriesList[0].dataProvider).toEqual("dp_1");
					expect(config.settings.charts[1].seriesList[0].id).toEqual("series_CLUSTER_cluster_5");
					expect(config.settings.charts[1].seriesList[0].name).toEqual("cluster Running reducers");
					expect(config.settings.charts[1].seriesList[0].type).toEqual("StepLine");
				});

				it("should now see a time series on chart 3.", function() {
					expect(config.settings.charts[2].id).toEqual("ACTIVITY_3001");
					expect(config.settings.charts[2].seriesList.length).toEqual(1);
					expect(config.settings.charts[2].seriesList[0].dataProvider).toEqual("dp_2");
					expect(config.settings.charts[2].seriesList[0].id).toEqual("series_ACTIVITY_job1_3001");
					expect(config.settings.charts[2].seriesList[0].name).toEqual("job1 Launched maps");
					expect(config.settings.charts[2].seriesList[0].type).toEqual("StepLine");
				});

				it("should now see a time series on chart 4.", function() {
					expect(config.settings.charts[3].id).toEqual("ACTIVITY_3004");
					expect(config.settings.charts[3].seriesList.length).toEqual(1);
					expect(config.settings.charts[3].seriesList[0].dataProvider).toEqual("dp_3");
					expect(config.settings.charts[3].seriesList[0].id).toEqual("series_ACTIVITY_job1_3004");
					expect(config.settings.charts[2].seriesList[0].name).toEqual("job1 Launched maps");
					expect(config.settings.charts[2].seriesList[0].type).toEqual("StepLine");
				});
			});
		});

		describe("ActivityChartSet.setContext Tests - Removal", function() {
			beforeEach(function() {
				jasmine.Ajax.useMock();
				chartSet.setContexts([{
					"id": "job1",
					"type": "ACTIVITY"
				}]);
			});

			describe("when cluster is removed", function() {
				it("should see 10 generalDataProviders.", function() {
					// We expect the configuration to have the following data providers.
					expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
					expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
					expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");

					expect(config.data.dataProviders.generalDataProviders[1].dataSet).toEqual("dataSet_1");
					expect(config.data.dataProviders.generalDataProviders[1].id).toEqual("dp_1");
				});

				it("should see 11 dataSets.", function() {
					expect(config.data.dataSets.length).toEqual(11);
					expect(config.data.dataSets[0].id).toEqual("dataSet_0");
					expect(config.data.dataSets[1].id).toEqual("dataSet_1");
					expect(config.data.dataSets[10].id).toEqual("chartStretcher");
				});

				it("should see 4 charts.", function() {
					expect(config.settings.charts.length).toEqual(4);
				});

				it("should see no time series on chart 1.", function() {
					expect(config.settings.charts[0].id).toEqual("CLUSTER_3");
					// This is because we don't remove series at the moment.
					// expect(config.settings.charts[0].seriesList.length).toEqual(0);
				});

				it("should see no time series on chart 2.", function() {
					expect(config.settings.charts[1].id).toEqual("CLUSTER_5");
					// This is because we don't remove chart series,
					// (we simply hide the chart series) on removeTimeSeries at the moment.
					// expect(config.settings.charts[1].seriesList.length).toEqual(0);
				});

				it("should see a time series on chart 3.", function() {
					expect(config.settings.charts[2].id).toEqual("ACTIVITY_3001");
					expect(config.settings.charts[2].seriesList.length).toEqual(1);
					expect(config.settings.charts[2].seriesList[0].dataProvider).toEqual("dp_2");
					expect(config.settings.charts[2].seriesList[0].id).toEqual("series_ACTIVITY_job1_3001");
					expect(config.settings.charts[2].seriesList[0].name).toEqual("job1 Launched maps");
					expect(config.settings.charts[2].seriesList[0].type).toEqual("StepLine");
				});

				it("should see a time series on chart 4.", function() {
					expect(config.settings.charts[3].id).toEqual("ACTIVITY_3004");
					expect(config.settings.charts[3].seriesList.length).toEqual(1);
					expect(config.settings.charts[3].seriesList[0].dataProvider).toEqual("dp_3");
					expect(config.settings.charts[3].seriesList[0].id).toEqual("series_ACTIVITY_job1_3004");
					expect(config.settings.charts[2].seriesList[0].name).toEqual("job1 Launched maps");
					expect(config.settings.charts[2].seriesList[0].type).toEqual("StepLine");
				});
			});
		});

		describe("ActivityChartSet.setChartDescriptors Tests - Change", function() {
			beforeEach(function() {
				jasmine.Ajax.useMock();
				var contexts = [{
					"id": "job1",
					"type": "ACTIVITY"
				}];

				var chartDescriptors = [{
					"metric" : {
						"id" : 22,
						"context" : "ACTIVITY",
						"name" : "disk_read",
						"desc" : "Reads from local disk (bytes / s)",
						"title" : "Disk reads",
						"selector" : "AVERAGE"
					},
					"chart" : {
						"id" : "ACTIVITY_22",
						"title" : "Disk reads",
						"discrete" : false
					}
				}];

				var skipRender = true;
				chartSet.setChartDescriptors(chartDescriptors, skipRender);
				chartSet.setContexts(contexts);
				// jQuery.publish("setChartDescriptors", [chartDescriptors]);

				// console.log(chartSet.getConfig());
			});

			it("should see 10 generalDataProviders.", function() {
				expect(config.data.dataProviders.generalDataProviders.length).toEqual(10);
				expect(config.data.dataProviders.generalDataProviders[0].dataSet).toEqual("dataSet_0");
				expect(config.data.dataProviders.generalDataProviders[0].id).toEqual("dp_0");
			});

			it("should see 11 dataSet.", function() {
				expect(config.data.dataSets.length).toEqual(11);
				expect(config.data.dataSets[0].id).toEqual("dataSet_0");
			});

			it("should see 1 chart.", function() {
				expect(config.settings.charts.length).toEqual(1);
			});

			it("should see 1 time series on chart 1.", function() {
				expect(config.settings.charts[0].id).toEqual("ACTIVITY_22");
				expect(config.settings.charts[0].seriesList.length).toEqual(1);
			});

			it("should see the result of _generateChartsPostData to match", function() {
				var queryString = chartSet._generateChartsPostData(new Date(1), new Date(10));
				var queryObject = Util.unparam(queryString);
				expect(queryObject.startTime).toEqual("1");
				expect(queryObject.endTime).toEqual("10");
				expect(queryObject["proxyContextMetrics[0].metric.metricId"]).toEqual("22");
				expect(queryObject["proxyContextMetrics[0].context.contextType"]).toEqual("ACTIVITY");
				expect(queryObject["proxyContextMetrics[0].context.contextId"]).toEqual("job1");
			});
			
			it("shoule see the result of _generateEventsPostData to match", function() {
				var queryString = chartSet._generateEventsPostData(new Date(1), new Date(10));
				var queryObject = Util.unparam(queryString);
				expect(queryObject.timeLower).toEqual("1");
				expect(queryObject.timeUpper).toEqual("10");
				expect(queryObject.summarise).toEqual("on");
				expect(queryObject.summariseGroupBy).toEqual("CATEGORY");
				expect(queryObject.summariseBy).toEqual("SEVERITY");
			});
		});
	});

	it("should see the result of getMetricParam to match", function() {
		var metric = {
			id : 2002,
			derivative: 'foo',
			scale: 'bar',
			units: 'foobar'
		};
		var params = chartSet.getMetricParams(0, metric, "CLUSTER", "cluster");
		expect(params).toEqual("proxyContextMetrics[0].metric.metricId=2002&proxyContextMetrics[0].metric.derivativeType=foo&proxyContextMetrics[0].metric.derivativeScale=bar&proxyContextMetrics[0].metric.units=foobar&proxyContextMetrics[0].context.contextType=CLUSTER&proxyContextMetrics[0].context.contextId=cluster");
	});
});
});
